@@ -1,11 +1,8 @@
-import logging
 import math
+from typing import Union
 
 import pandas as pd
 
-logging.basicConfig()
-logger = logging.getLogger(__name__)
-
 
 def nbytes(df):
     try:
@@ -15,11 +12,32 @@ def nbytes(df):
         return df.estimated_size()
 
 
-def downsample(df, max_rows=0, max_columns=0, max_bytes=0):
+def as_nbytes(mem):
+    if isinstance(mem, (int, float)):
+        return int(mem)
+    assert isinstance(mem, str), mem
+    if mem.endswith("KB"):
+        return int(float(mem[:-2]) * 2**10)
+    if mem.endswith("MB"):
+        return int(float(mem[:-2]) * 2**20)
+    if mem.endswith("GB"):
+        raise ValueError(
+            f"You probably don't want to display "
+            f"a table that large within an HTML page: {mem}"
+        )
+    if mem.endswith("B"):
+        return int(float(mem[:-1]))
+    return int(float(mem))
+
+
+def downsample(
+    df, max_rows: int = 0, max_columns: int = 0, max_bytes: Union[int, str] = 0
+):
     """Return a subset of the dataframe that fits the limits"""
     org_rows, org_columns, org_bytes = len(df), len(df.columns), nbytes(df)
+    max_bytes_numeric = as_nbytes(max_bytes)
     df = _downsample(
-        df, max_rows=max_rows, max_columns=max_columns, max_bytes=max_bytes
+        df, max_rows=max_rows, max_columns=max_columns, max_bytes=max_bytes_numeric
     )
 
     if len(df) < org_rows or len(df.columns) < org_columns:
@@ -29,7 +47,7 @@ def downsample(df, max_rows=0, max_columns=0, max_bytes=0):
             reasons.append("maxRows={}".format(max_rows))
         if org_columns > max_columns > 0:
             reasons.append("maxColumns={}".format(max_columns))
-        if org_bytes > max_bytes > 0:
+        if org_bytes > max_bytes_numeric > 0:
             reasons.append("maxBytes={}".format(max_bytes))
 
         warning = "{} from {:,d}x{:,d} to {:,d}x{:,d} as {}".format(
@@ -79,6 +97,7 @@ def _downsample(df, max_rows=0, max_columns=0, max_bytes=0, target_aspect_ratio=
     if len(df) > max_rows > 0:
         second_half = max_rows // 2
         first_half = max_rows - second_half
+        assert first_half >= second_half
         if second_half:
             try:
                 df = pd.concat((df.iloc[:first_half], df.iloc[-second_half:]))
@@ -93,17 +112,18 @@ def _downsample(df, max_rows=0, max_columns=0, max_bytes=0, target_aspect_ratio=
     if len(df.columns) > max_columns > 0:
         second_half = max_columns // 2
         first_half = max_columns - second_half
+        assert first_half >= second_half
         if second_half:
-            try:
+            if isinstance(df, pd.DataFrame):
                 df = pd.concat(
                     (df.iloc[:, :first_half], df.iloc[:, -second_half:]), axis=1
                 )
-            except AttributeError:
+            else:
                 df = df[df.columns[:first_half]].hstack(df[df.columns[-second_half:]])
         else:
-            try:
+            if isinstance(df, pd.DataFrame):
                 df = df.iloc[:, :first_half]
-            except AttributeError:
+            else:
                 df = df[df.columns[:first_half]]
 
     df_nbytes = nbytes(df)
@@ -127,12 +147,14 @@ def _downsample(df, max_rows=0, max_columns=0, max_bytes=0, target_aspect_ratio=
             )
 
         # max_bytes is smaller than the average size of one cell
-        try:
-            df = df.iloc[:1, :1]
-            df.iloc[0, 0] = "..."
-        except AttributeError:
+        if isinstance(df, pd.DataFrame):
+            return pd.DataFrame("...", index=df.index[:1], columns=df.columns[:1])
+
+        else:
             import polars as pl  # noqa
 
+            assert isinstance(df, pl.DataFrame)
+
             df = pl.DataFrame({df.columns[0]: ["..."]})
         return df
 
